d = 50; % dimension of the data

M = 20;  % degree of the graph

N = 2 * 100;  % total number of training samples

r = 10; % number of edges sampled for each node

alpha = 1.2; % improvement ratio compared with random sampling

p_ratio = 0.5; % pruning rate

N_p = N / 2; % number of positive data

N_g = N / 2; % number of negative data

K = 200; % 2*K = number of neurons

N_test = 20000; % number of test data

L = 50; % number of discriminative and non-discriminative patterns

T = 500; % number of maximum iterations

mag = 0.5; % magnitude of each pattern

sigma = 0.2; % noise level

O = generate_O( d , L, mag, sigma );

[ X, Y , ~ ] = generate_GX( N , O , M ); % generate the input vector

W_0 = 1 / 10 * randn( d , K ); % initilize W

V_0 = 1 / 10 * randn( d , K ); % initilize W

eta = 0.5 * K; % stepsize

training_error = nan( T , 2 ); % record of training error


% Trainging start

W = W_0;

V = V_0;

for t = 1 : 20
    
    X_s = sample_MR( X , M, r , alpha ); % random sampling
    
    [ g_W , g_V ] = Gradient_GNN( W , V , X_s , Y);
    
    W = W - eta * g_W;

    V = V - eta * g_V;
    
end

fprintf('Sampling rate = %.2f, pruning rate = %.2f\n', r/M, p_ratio);

fprintf( 'Loop Number =        ' );

temp_1 = sort( diag(W'*W) );

thr = floor( K * p_ratio ); 

thr_v = temp_1( thr );

W = W_0( : , ( diag(W'*W) > thr_v ) ); 

temp_2 = sort( diag(V'*V) );

thr_v = temp_2( thr );

V = V_0( : , ( diag(V'*V) > thr_v ) ); 

for t = 1 : T
    
    fprintf('\b\b\b\b\b\b\b%6d\n', t);
    
    X_s = sample_MR( X , M, r , alpha ); % random sampling
    
    [ g_W , g_V ] = Gradient_GNN( W , V , X_s , Y);
    
    W = W - eta * g_W;

    V = V - eta * g_V;
    
    error_t =  mean( max( 1 -  generate_y( W , V , X).* Y , 0) ); 
    
    training_error( t , 1 ) = error_t;
        
    if error_t == 0 || ( norm( g_W , 'fro' ) == 0 && norm( g_V , 'fro' ) == 0 )
        
        break;
        
    end

end


% plot the error

plot( 1 : T , training_error );

xlabel('Number of iterations');

ylabel('Training loss');

legend('Model after magnitude pruning','Original model');

[ X_test , Y_test ] = generate_GX( N , O , M );

Y_est = generate_y( W , V , X_test );

test_error = mean(  sign(Y_est) ~= sign( Y_test ) );
 
fprintf('The test error = %f\n' , test_error );







